@@ -12,15 +12,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from typing import Set, Callable, List
+from typing import List
 
 import json
 import ssl
 import threading
 import logging
 
-import base64
-
 import websocket as ws
 
 from ._reader import QuoteReader
@@ -29,22 +27,6 @@ __all__ = ['YAHOO_FINANCE_SOCKET', 'QuoteStreamer']
 
 _logger = logging.getLogger("yflive")
 
-# ==============================================================================
-# Default callback methods
-# ==============================================================================
-
-def _on_connect(qs):
-    pass
-
-def _on_quote(qs, quote):
-    pass
-
-def _on_error(qs, error):
-    pass
-
-def _on_close(qs):
-    pass
-
 # ==============================================================================
 # QuoteStreamer
 # ==============================================================================
@@ -62,10 +44,10 @@ class QuoteStreamer:
     subscribed to, which are then parced.
     """
 
-    on_connect = _on_connect
-    on_quote = _on_quote
-    on_error = _on_error
-    on_close = _on_close
+    on_connect = None
+    on_quote = None
+    on_error = None
+    on_close = None
 
     instance = None
 
@@ -114,7 +96,11 @@ class QuoteStreamer:
         if not self.streaming: 
             return
         _logger.debug("Stopping QuoteStreamer...")
-        self._close()
+        if isinstance(self._websocket, ws.WebSocketApp):
+            self._websocket.close()
+            self._websocket = None
+        if self._ws_thread:
+            self._ws_thread.join()
 
     def _run(self):
         try:
@@ -126,21 +112,13 @@ class QuoteStreamer:
                         on_open = _ws_open)
             self._websocket.run_forever(sslopt={"cert_reqs": ssl.CERT_NONE})
         except (Exception, KeyboardInterrupt, SystemExit) as e:
-            self.on_error(e)
+            self._callback(self.on_error, (e))
             if isinstance(e, SystemExit):
                 # propagate SystemExit further
                 raise
             self.stop()
             return not isinstance(e, KeyboardInterrupt)
 
-
-    def _close(self):
-        if isinstance(self._websocket, ws.WebSocketApp):
-            self._websocket.close()
-            self._websocket = None
-        if self._ws_thread:
-            self._ws_thread.join()
-
     @property
     def subscribed(self) -> List:
         """Get all currently tracked identifiers."""
@@ -151,7 +129,7 @@ class QuoteStreamer:
         """Get current streaming state."""
         return self._websocket is not None
 
-    def subscribe(self, identifiers=[]):
+    def subscribe(self, identifiers=None):
         """
         Subscribe to identifiers.
         
@@ -160,17 +138,17 @@ class QuoteStreamer:
         identifiers: <type>
             identifiers to subscribe to
         """
-        identifiers = set(identifiers) - self._subscribed
-        if len(identifiers) <= 0: 
+        if identifiers is None and len(identifiers) <= 0:
             return
+        identifiers = set(identifiers) - self._subscribed
         self._subscribed = self._subscribed | identifiers
         _logger.debug(f"Subscribing to {list(identifiers)}")
-        if self.streaming:
+        if self.streaming and len(identifiers) > 0:
             msg = json.dumps({"subscribe": list(identifiers)})
             self._websocket.send(msg)
         return
 
-    def unsubscribe(self, identifiers=[]):
+    def unsubscribe(self, identifiers=None):
         """
         Unsubscribe from identifiers.
         
@@ -179,9 +157,9 @@ class QuoteStreamer:
         identifiers: <type>
             identifiers to unsubscribe from
         """
-        identifiers = self._subscribed & set(identifiers)
-        if len(identifiers) <= 0: 
+        if identifiers is None and len(identifiers) <= 0:
             return
+        identifiers = self._subscribed & set(identifiers)
         self._subscribed = self._subscribed - identifiers
         _logger.debug(f"Unsubscribing from {list(identifiers)}")
         if self.streaming and len(identifiers) > 0:
@@ -189,6 +167,13 @@ class QuoteStreamer:
             self._websocket.send(msg)
         return
 
+    def _callback(self, callback, *args):
+        if callback and callable(callback):
+            try:
+                callback(self, *args)
+            except Exception as e:
+                _logger.error("error from callback {}: {}".format(callback, e))
+
 # ==============================================================================
 # Websocket callback methods
 # ==============================================================================
@@ -197,8 +182,7 @@ _streamer = QuoteStreamer()
 
 def _ws_open(ws):
     _logger.debug("Yahoo! Finance connection opened")
-    if _streamer.on_connect:
-        _streamer.on_connect()
+    _streamer._callback(_streamer.on_connect)
     if len(_streamer.subscribed) > 0:
         msg = json.dumps({"subscribe": list(_streamer.subscribed)})
         ws.send(msg)
@@ -206,16 +190,13 @@ def _ws_open(ws):
 def _ws_message(ws, message):
     quote = QuoteReader.parse(message)
     _logger.info(f"Quote received: {str(quote)}")
-    if _streamer.on_quote:
-        _streamer.on_quote(quote)
+    _streamer._callback(_streamer.on_quote, (quote))
     
 def _ws_error(ws, error):
     _logger.error(f"Error encountered: {error}")
-    if _streamer.on_error:
-        _streamer.on_error(error)
+    _streamer._callback(_streamer.on_error, (error))
 
 def _ws_close(ws):
     _logger.debug("Connection closed")
-    if _streamer.on_close:
-        _streamer.on_close()
-    _streamer._close()
\ No newline at end of file
+    _streamer._callback(_streamer.on_close)
+    _streamer.stop()
\ No newline at end of file
